import { LineTimeSerie, LineOptions, LinePod } from '@chartwerk/line-pod';

import { AxisRange } from '@chartwerk/core/dist/types';
import { MarkersConf } from '@chartwerk/line-pod/dist/models/marker';
import { SegmentSerie } from '@chartwerk/line-pod/dist/models/segment';

import { useEffect, useRef, useState } from 'react';

import _ from 'lodash';


export type ChartwerkLinePodProps = {
  id?: string;
  series: LineTimeSerie[];
  options?: LineOptions;
  markersConf?: MarkersConf,
  segments?: SegmentSerie[],
  className?: string;
  // TODO: callback types should be exported from chartwerk
  onZoomIn?: (ranges: AxisRange[]) => void;
  onZoomOut?: (centers: { x: number, y: number }) => void;
  onMouseMove?: (event: any) => void;
  onMouseOut?: () => void;
  onLegendClick?: (idx: number) => void,
  onPanning?: (event: { ranges: AxisRange[], d3Event: any }) => void;
  onPanningEnd?: (ranges: AxisRange[]) => void;
  onContextMenu?: (evt: any) => void;
  onSharedCrosshairMove?: (evt: any) => void;
  onRenderStart?: () => void,
  onRenderEnd?: () => void,
}

export function ChartwerkLinePod(props: ChartwerkLinePodProps) {
  
  const [pod, setPod] = useState<LinePod | null>(null);
  const [hack, setHack] = useState<number | null>(null);

  const chartRef = useRef(null);
  const chart = chartRef.current;

  useEffect(() => {
    // this function will be called on component unmount
    return () => {
      if(pod === null) { return; }
      // @ts-ignore
      pod.removeEventListeners();
    }
  }, []);

  useEffect(() => {
    if(chart === null) { return; }

    let eventsCallbacks = _.cloneDeep(props.options?.eventsCallbacks || {});
    if (props.onZoomIn) {
      eventsCallbacks.zoomIn = props.onZoomIn;
    }
    if (props.onZoomOut) {
      eventsCallbacks.zoomOut = props.onZoomOut;
    }
    if (props.onMouseMove) {
      eventsCallbacks.mouseMove = props.onMouseMove;
    }
    if (props.onMouseOut) {
      eventsCallbacks.mouseOut = props.onMouseOut;
    }
    if (props.onLegendClick) {
      eventsCallbacks.onLegendClick = props.onLegendClick;
    }
    if (props.onPanning) {
      eventsCallbacks.panning = props.onPanning;
    }
    if (props.onPanningEnd) {
      eventsCallbacks.panningEnd = props.onPanningEnd;
    }
    if (props.onContextMenu) {
      eventsCallbacks.contextMenu = props.onContextMenu;
    }
    if (props.onSharedCrosshairMove) {
      eventsCallbacks.sharedCrosshairMove = props.onSharedCrosshairMove;
    }
    if (props.onRenderStart) {
      eventsCallbacks.renderStart = props.onRenderStart;
    }

    if(pod === null) {
      const newPod = new LinePod(
        // @ts-ignore
        chart,
        props.series,
        {
          ...props.options,
          eventsCallbacks
        },
        props.markersConf,
        props.segments
      );
      setPod(newPod);
      newPod.render();
    } else {
      pod.updateData(props.series, props.options);
    }
  }, [chart, pod, props.id, props.options, props.markersConf, props.segments]);

  // TODO: it's a hack to render the LinePod right after the div appears in DOM
  setTimeout(() => {
    if(hack === null) {
      setHack(1);
    }
  }, 1);


  return (
    <div id={props.id} className={props.className} ref={chartRef}></div>
  );
}

export default ChartwerkLinePod;
